# -*- coding: utf-8 -*-
#
# This file is part of the CspMaster project
#
# INAF - SKA Telescope
#
# Distributed under the terms of the GPL license.
# See LICENSE.txt for more info.

""" CSP.LMC Common CspMaster

CSP.LMC Common Class for the CSPMaster TANGO Device.
"""

# tango imports
import tango
from tango import DebugIt
from tango.server import run
from tango.server import Device, DeviceMeta
from tango.server import attribute, command
from tango.server import device_property
from tango import AttrQuality, DispLevel, DevState
from tango import AttrWriteType, PipeWriteType
from skabase.SKAMaster  import SKAMaster
# Additional import
# PROTECTED REGION ID(CspMaster.additionnal_import) ENABLED START #
# PROTECTED REGION END #    //  CspMaster.additionnal_import

__all__ = ["CspMaster", "main"]


class CspMaster(SKAMaster):
    """
    CSP.LMC Common Class for the CSPMaster TANGO Device.

    **Properties:**

    - Device Property
    
        CspCbf
            - FQDN of the  CBF Sub-element Master
            - Type:'DevString'
    
        CspPss
            - TANGO FQDN of the  PSS Sub-element Master
            - Type:'DevString'
    
        CspPst
            - TANGO FQDN of the  PST Sub-element Master
            - Type:'DevString'
    
        CspSubarrays
            - TANGO FQDN of the  CSP.LMC Subarrays
            - Type:'DevVarStringArray'
    
        SearchBeams
            - TANGO FQDN of the CSP.LMC SearchBeam Capabilities
            - Type:'DevVarStringArray'
    
        TimingBeams
            - TANGO FQDN of the CSP.LMC TimingBeam Capabilities
            - Type:'DevVarStringArray'
    
        VlbiBeams
            - TANGO FQDN of the CSP.LMC Vlbi Capabilities
            - Type:'DevVarStringArray'
    
    """
    __metaclass__ = DeviceMeta
    # PROTECTED REGION ID(CspMaster.class_variable) ENABLED START #
    # PROTECTED REGION END #    //  CspMaster.class_variable

    # -----------------
    # Device Properties
    # -----------------

    CspCbf = device_property(
        dtype='DevString',
    )

    CspPss = device_property(
        dtype='DevString',
    )

    CspPst = device_property(
        dtype='DevString',
    )

    CspSubarrays = device_property(
        dtype='DevVarStringArray',
    )

    SearchBeams = device_property(
        dtype='DevVarStringArray',
    )

    TimingBeams = device_property(
        dtype='DevVarStringArray',
    )

    VlbiBeams = device_property(
        dtype='DevVarStringArray',
    )

    # ----------
    # Attributes
    # ----------

    adminMode = attribute(
        dtype='DevEnum',
        access=AttrWriteType.READ_WRITE,
        polling_period=1000,
        memorized=True,
        doc= ("The admin mode reported for this device. It may interpret the" 
              "current device condition and condition of all managed devices"
              "to set this. Most possibly an aggregate attribute."),
    )

    onCommandProgress = attribute(
        dtype='DevUShort',
        label="Progress percentage for the On command",
        polling_period=3000,
        abs_change=10,
        max_value=100,
        min_value=0,
        doc=("Percentage progress implemented for commands that  result in state/mode"
             "transitions for a large \nnumber of components and/or are executed in "
             "stages (e.g power up, power down)"),
    )

    offCommandProgress = attribute(
        dtype='DevUShort',
        label="Progress percentage for the Off command",
        polling_period=3000,
        abs_change=10,
        max_value=100,
        min_value=0,
        doc=("Percentage progress implemented for commands that  result in state/mode"
             " transitions for a large \nnumber of components and/or are executed in "
             "stages (e.g power up, power down)"),
    )

    standbyCommandProgress = attribute(
        dtype='DevUShort',
        label="Progress percentage for the Standby command",
        polling_period=3000,
        abs_change=10,
        max_value=100,
        min_value=0,
        doc=("Percentage progress implemented for commands that  result in state/mode "
            "transitions for a large \nnumber of components and/or are executed in "
            "stages (e.g power up, power down)"),
    )

    onCmdDurationExpected = attribute(
        dtype='DevUShort',
        access=AttrWriteType.READ_WRITE,
        label="Expected duration (sec) of the On command execution",
        polling_period=3000,
        abs_change=0,
        max_value=100,
        min_value=0,
        memorized=True,
        doc=("Set/Report the duration expected (in sec.) for the On command execution"),
    )

    offCmdDurationExpected = attribute(
        dtype='DevUShort',
        access=AttrWriteType.READ_WRITE,
        label="Expected duration (sec) of the Off command",
        polling_period=3000,
        abs_change=0,
        max_value=100,
        min_value=0,
        memorized=True,
        doc=("Set/Report the duration expected (in sec.) for the Off command execution"),
    )

    standbyCmdDurationExpected = attribute(
        dtype='DevUShort',
        access=AttrWriteType.READ_WRITE,
        label="Expected duration (sec) of the Standby command",
        polling_period=3000,
        abs_change=0,
        max_value=100,
        min_value=0,
        memorized=True,
        doc=("Set/Report the duration expected (in sec.) for the Standby command"),
    )

    onCmdDurationMeasured = attribute(
        dtype='DevUShort',
        label="Measured duration (sec) of the On command execution",
        polling_period=3000,
        abs_change=0,
        max_value=100,
        min_value=0,
        doc=("Report the measured duration (in sec.) of the On command execution"),
    )

    offCmdDurationMeasured = attribute(
        dtype='DevUShort',
        label="Measured duration (sec) of the Off command",
        polling_period=3000,
        abs_change=0,
        max_value=100,
        min_value=0,
        doc=("Report the measured duration (in sec.) of the Off command execution"),
    )

    standbyCmdDurationMeasured = attribute(
        dtype='DevUShort',
        label="Measured duration (sec) of the Standby command",
        polling_period=3000,
        abs_change=0,
        max_value=100,
        min_value=0,
        doc=("Report the measured duration (in sec.) of the Standby command"),
    )

    cbfOnCmdTimeoutExpired = attribute(
        dtype='DevBoolean',
        label="CBF command timeout flag",
        polling_period=2000,
        abs_change=1,
        doc=("Signal the occurence of a timeout during the execution of commands on"
             " CBF Sub-element"),
    )

    pssOnCmdTimeoutExpired = attribute(
        dtype='DevBoolean',
        label="PSS command timeout flag",
        polling_period=2000,
        abs_change=1,
        doc=("Signal the occurence of a timeout during the execution of commands on "
             "PSS Sub-element"),
    )

    pstOnCmdTimeoutExpired = attribute(
        dtype='DevBoolean',
        label="PST command timeout flag",
        polling_period=2000,
        abs_change=1,
        doc=("Signal the occurence of a timeout during the execution of commands on"
             " PST Sub-element"),
    )

    cspCbfState = attribute(
        dtype='DevState',
        label="CBF status",
        polling_period=3000,
        abs_change=1,
        doc=("The CBF sub-element Device State. Allowed values are ON, STANDBY, OFF, "
             "DISABLED, ALARM, FAULT, UNKNOWN"),
    )

    cspPssState = attribute(
        dtype='DevState',
        label="PSS status",
        polling_period=3000,
        abs_change=1,
        doc=("The PSS sub-element Device State. Allowed values are ON, STANDBY, OFF,"
             " DISABLED, ALARM, FAULT, UNKNOWN"),
    )

    cspPstState = attribute(
        dtype='DevState',
        label="PST status",
        polling_period=3000,
        doc=("The PST sub-element Device State. Allowed values are ON, STANDBY,OFF,"
             " DISABLED, ALARM, FAULT, UNKNOWN"),
    )

    cspCbfHealthState = attribute(
        dtype='DevEnum',
        label="CBF Health status",
        polling_period=3000,
        abs_change=1,
        doc="The CBF sub-element healthState.",
        enum_labels=["OK", "DEGRADED", "FAILED", "UNKNOWN", ],
    )

    cspPssHealthState = attribute(
        dtype='DevEnum',
        label="PSS Health status",
        polling_period=3000,
        abs_change=1,
        doc="The PSS sub-element healthState",
        enum_labels=["OK", "DEGRADED", "FAILED", "UNKNOWN", ],
    )

    cspPstHealthState = attribute(
        dtype='DevEnum',
        label="PST health status",
        polling_period=3000,
        abs_change=1,
        doc="The PST sub-element healthState.",
        enum_labels=["OK", "DEGRADED", "FAILED", "UNKNOWN", ],
    )

    cbfMasterAddress = attribute(
        dtype='DevString',
        doc="TANGO FQDN of the CBF sub-element Master",
    )

    pssMasterAddress = attribute(
        dtype='DevString',
        doc="TANGO FQDN of the PSS sub-element Master",
    )

    pstMasterAddress = attribute(
        dtype='DevString',
        doc="TANGO FQDN of the PST sub-element Master",
    )

    cspCbfAdminMode = attribute(
        dtype='DevEnum',
        access=AttrWriteType.READ_WRITE,
        label="CBF administrative Mode",
        polling_period=3000,
        abs_change=1,
        doc=("The CBF sub-lement adminMode. Allowed values are ON-LINE, "
             "MAINTENANCE, OFF-LINE, NOT-FITTED, RESERVED")
    )

    cspPssAdminMode = attribute(
        dtype='DevEnum',
        access=AttrWriteType.READ_WRITE,
        label="PSS administrative mode",
        polling_period=3000,
        abs_change=1,
        doc=("The PSS sub-lement adminMode. Allowed values are ON-LINE, "
             "MAINTENANCE, OFF-LINE, NOT-FITTED, RESERVED")
    )

    cspPstAdminMode = attribute(
        dtype='DevEnum',
        access=AttrWriteType.READ_WRITE,
        label="PST administrative mode",
        polling_period=3000,
        abs_change=1,
        doc=("The PST sub-lement adminMode. Allowed values are ON-LINE, "
             "MAINTENANCE, OFF-LINE, NOT-FITTED, RESERVED")
    )

    unassignedSearchBeamsNum = attribute(
        dtype='DevUShort',
        label="Number of unassigned SearchBeam Capabilities",
        doc=("Report the number of unassigned SearchBeam Capabilities."
             " This number does not take in account the number of *reserved*"
             " SearchBeams."),
    )


    availableCapabilities = attribute(
        dtype=('DevString',),
        max_dim_x=20,
        doc=("A list of available number of instances of each capability type, e.g."
             " `CORRELATOR:512`, `PSS-BEAMS:4`."),
    )

    reportSearchBeamState = attribute(
        dtype=('DevState',),
        max_dim_x=1500,
        label="SearchBeam Capabilities State",
        doc="Report the state of the SearchBeam Capabilities as an array of DevState.",
    )

    reportSearchBeamHealthState = attribute(
        dtype=('DevUShort',),
        max_dim_x=1500,
        label="SearchBeam Capabilities healthState",
        abs_change=1,
        doc=("Report the health state of the SearchBeam Capabilities as an array"
             " of unsigned short. For ex`: [0,0,...,1..]`"),
    )

    reportSearchBeamAdminMode = attribute(
        dtype=('DevUShort',),
        access=AttrWriteType.READ_WRITE,
        max_dim_x=1500,
        label="SearchBeam Capabilities adminMode",
        abs_change=1,
        doc=("Report the administrative mode of the SearchBeam Capabilities as an"
             " array of unsigned short. For ex `:[0,0,0,...2..]`"),
    )

    reportTimingBeamState = attribute(
        dtype=('DevState',),
        max_dim_x=16,
        label="TimingBeam Capabilities State",
        doc="Report the state of the TimingBeam Capabilities as an array of DevState.",
    )

    reportTimingBeamHealthState = attribute(
        dtype=('DevUShort',),
        max_dim_x=16,
        label="TimingBeam Capabilities healthState",
        abs_change=1,
        doc=("Report the health state of the TimingBeam Capabilities as an array of"
             " unsigned short. For ex [0,0,...,1..]"),
    )

    reportTimingBeamAdminMode = attribute(
        dtype=('DevUShort',),
        access=AttrWriteType.READ_WRITE,
        max_dim_x=16,
        label="TimingBeam Capabilities adminMode",
        abs_change=1,
        doc=("Report the administrativw mode of the TimingBeam Capabilities as an"
             " array of unsigned short. For ex [0,0,0,...2..]"),
    )

    reportVlbiBeamState = attribute(
        dtype=('DevState',),
        max_dim_x=20,
        label="VlbiBeam Capabilities State",
        doc="Report the state of the VLBIBeam Capabilities as an array of DevState.",
    )

    reportVlbiBeamHealthState = attribute(
        dtype=('DevUShort',),
        max_dim_x=20,
        label="VlbiBeam Capabilities healthState",
        abs_change=1,
        doc=("Report the health state of the VlbiBeam Capabilities as an array of"
             " unsigned short. For ex [0,0,...,1..]"),
    )

    reportVlbiBeamAdminMode = attribute(
        dtype=('DevUShort',),
        access=AttrWriteType.READ_WRITE,
        max_dim_x=20,
        label="VlbiBeam Capabilities adminMode",
        abs_change=1,
        doc=("Report the administrative mode of the VlbiBeam Capabilities as an"
             " array of unsigned short. For ex -[0,0,0,...2..]"),
    )

    cspSubarrayAddresses = attribute(
        dtype=('DevString',),
        max_dim_x=16,
        doc="CSPSubarrays FQDN",
    )

    searchBeamAddresses = attribute(
        dtype=('DevString',),
        max_dim_x=1500,
        label="SearchBeamCapabilities FQDNs",
        doc="TANGO FQDNs of the CSP.LMC SearchBeam Capabilities",
    )

    timingBeamAddresses = attribute(
        dtype=('DevString',),
        max_dim_x=16,
        label="TimingBeam Capabilities FQDN",
        doc="TANGO FQDNs of the CSP.LMC TimingBeam Capabilities",
    )

    vlbiBeamAddresses = attribute(
        dtype=('DevString',),
        max_dim_x=20,
        label="VLBIBeam Capabilities FQDNs",
        doc="TANGO FQDNs of the CSP.LMC VlbiBeam Capabilities",
    )

    searchBeamMembership = attribute(
        dtype=('DevUShort',),
        max_dim_x=1500,
        label="SearchBeam Memebership",
        doc="Reports the sub-array affiliation for each SearchBeam Capabiity.",
    )

    timingBeamMembership = attribute(
        dtype=('DevUShort',),
        max_dim_x=16,
        label="TimingBeam Membership",
        doc="Report the sub-array affiliation for each TimingBeam Capability.",
    )

    vlbiBeamMembership = attribute(
        dtype=('DevUShort',),
        max_dim_x=20,
        label="VLBI Beam membership",
        doc="Report the sub-array affiliation for each VlbiBbeam Capability.",
    )

    unassignedSearchBeamIDs = attribute(
        dtype=('DevUShort',),
        max_dim_x=1500,
        label="Unassigned SeachBeam Capabilities IDs",
        doc=("Report the list of SearchBeam Capability IDs that are not assigned to any sub-array."
             "SearchBeams are assigned to a sub-array in groups of three because each PSS LRU "
             "can process up to three SearchBeams at time. A SearchBeam is marked as *reserved* to"
             " a sub-array if the other SearchBeams of the group are already assigned to that sub-array."
             " The IDs of the SearchBeams marked as *reserved*, are not included in this list."
             " The SCM values of a unassigned SearchBeam Capabiity are "
             " State = OFF adminMode = ONLINE or MAINTENACE obsState = IDLE healthState = OK "
             "obsMode = IDLE (reserved flag = False)"),
    )

    reservedSearchBeamIDs = attribute(
        dtype=('DevUShort',),
        max_dim_x=16,
        label="IDs of reserved SeachBeam Capabilities",
        doc="Report the list of SearchBeam Capability IDs reserved for each sub-array",
    )

    reservedSearchBeamNum = attribute(
        dtype=('DevUShort',),
        max_dim_x=16,
        label="Number of reserved SeachBeam Capabilities",
        doc="Report the number of SearchBeam Capabilities reserved for each sub-array",
    )

    unassignedTimingBeamIDs = attribute(
        dtype=('DevUShort',),
        max_dim_x=16,
        label="Unassigned TimingBeam Capabilities IDs",
        doc="Report the list of TimingBeam Capability IDs that are not assigned to any sub-array. The SCM values of a unassigned VlbiBeam Capabiity are:\nState = OFF\nadminMode = ONLINE or MAINTENACE\nobsState = IDLE\nhealthState = OK \nobsMode = IDLE\n",
    )

    unassignedVlbiBeamIDs = attribute(
        dtype=('DevUShort',),
        max_dim_x=20,
        label="Unassigned VlbiBeam Capabilities IDs",
        doc="Report the list of VlbiBeam Capability IDs that are not assigned to any sub-array. The SCM values of a unassigned VlbiBeam Capabiity are:\nState = OFF\nadminMode = ONLINE or MAINTENACE\nobsState = IDLE\nhealthState = OK \nobsMode = IDLE\n",
    )

    # ---------------
    # General methods
    # ---------------

    def init_device(self):
        """Initialises the attributes and properties of the CspMaster."""
        SKAMaster.init_device(self)       
        # PROTECTED REGION ID(CspMaster.init_device) ENABLED START #
        # PROTECTED REGION END #    //  CspMaster.init_device

    def always_executed_hook(self):
        """Method always executed before any TANGO command is executed."""
        # PROTECTED REGION ID(CspMaster.always_executed_hook) ENABLED START #
        # PROTECTED REGION END #    //  CspMaster.always_executed_hook

    def delete_device(self):
        """Hook to delete resources allocated in init_device.

        This method allows for any memory or other resources allocated in the
        init_device method to be released.  This method is called by the device
        destructor and by the device Init command.
        """
        # PROTECTED REGION ID(CspMaster.delete_device) ENABLED START #
        # PROTECTED REGION END #    //  CspMaster.delete_device
    # ------------------
    # Attributes methods
    # ------------------

    def read_adminMode(self):
        # PROTECTED REGION ID(CspMaster.adminMode_read) ENABLED START #
        """Return the adminMode attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.adminMode_read

    def write_adminMode(self, value):
        # PROTECTED REGION ID(CspMaster.adminMode_write) ENABLED START #
        """Set the adminMode attribute."""
        pass
        # PROTECTED REGION END #    //  CspMaster.adminMode_write

    def read_onCommandProgress(self):
        # PROTECTED REGION ID(CspMaster.onCommandProgress_read) ENABLED START #
        """Return the onCommandProgress attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.onCommandProgress_read

    def read_offCommandProgress(self):
        # PROTECTED REGION ID(CspMaster.offCommandProgress_read) ENABLED START #
        """Return the offCommandProgress attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.offCommandProgress_read

    def read_standbyCommandProgress(self):
        # PROTECTED REGION ID(CspMaster.standbyCommandProgress_read) ENABLED START #
        """Return the standbyCommandProgress attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.standbyCommandProgress_read

    def read_onCmdDurationExpected(self):
        # PROTECTED REGION ID(CspMaster.onCmdDurationExpected_read) ENABLED START #
        """Return the onCmdDurationExpected attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.onCmdDurationExpected_read

    def write_onCmdDurationExpected(self, value):
        # PROTECTED REGION ID(CspMaster.onCmdDurationExpected_write) ENABLED START #
        """Set the onCmdDurationExpected attribute."""
        pass
        # PROTECTED REGION END #    //  CspMaster.onCmdDurationExpected_write

    def read_offCmdDurationExpected(self):
        # PROTECTED REGION ID(CspMaster.offCmdDurationExpected_read) ENABLED START #
        """Return the offCmdDurationExpected attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.offCmdDurationExpected_read

    def write_offCmdDurationExpected(self, value):
        # PROTECTED REGION ID(CspMaster.offCmdDurationExpected_write) ENABLED START #
        """Set the offCmdDurationExpected attribute."""
        pass
        # PROTECTED REGION END #    //  CspMaster.offCmdDurationExpected_write

    def read_standbyCmdDurationExpected(self):
        # PROTECTED REGION ID(CspMaster.standbyCmdDurationExpected_read) ENABLED START #
        """Return the standbyCmdDurationExpected attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.standbyCmdDurationExpected_read

    def write_standbyCmdDurationExpected(self, value):
        # PROTECTED REGION ID(CspMaster.standbyCmdDurationExpected_write) ENABLED START #
        """Set the standbyCmdDurationExpected attribute."""
        pass
        # PROTECTED REGION END #    //  CspMaster.standbyCmdDurationExpected_write

    def read_onCmdDurationMeasured(self):
        # PROTECTED REGION ID(CspMaster.onCmdDurationMeasured_read) ENABLED START #
        """Return the onCmdDurationMeasured attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.onCmdDurationMeasured_read

    def read_offCmdDurationMeasured(self):
        # PROTECTED REGION ID(CspMaster.offCmdDurationMeasured_read) ENABLED START #
        """Return the offCmdDurationMeasured attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.offCmdDurationMeasured_read

    def read_standbyCmdDurationMeasured(self):
        # PROTECTED REGION ID(CspMaster.standbyCmdDurationMeasured_read) ENABLED START #
        """Return the standbyCmdDurationMeasured attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.standbyCmdDurationMeasured_read

    def read_cbfOnCmdTimeoutExpired(self):
        # PROTECTED REGION ID(CspMaster.cbfOnCmdTimeoutExpired_read) ENABLED START #
        """Return the cbfOnCmdTimeoutExpired attribute."""
        return False
        # PROTECTED REGION END #    //  CspMaster.cbfOnCmdTimeoutExpired_read

    def read_pssOnCmdTimeoutExpired(self):
        # PROTECTED REGION ID(CspMaster.pssOnCmdTimeoutExpired_read) ENABLED START #
        """Return the pssOnCmdTimeoutExpired attribute."""
        return False
        # PROTECTED REGION END #    //  CspMaster.pssOnCmdTimeoutExpired_read

    def read_pstOnCmdTimeoutExpired(self):
        # PROTECTED REGION ID(CspMaster.pstOnCmdTimeoutExpired_read) ENABLED START #
        """Return the pstOnCmdTimeoutExpired attribute."""
        return False
        # PROTECTED REGION END #    //  CspMaster.pstOnCmdTimeoutExpired_read

    def read_cspCbfState(self):
        # PROTECTED REGION ID(CspMaster.cspCbfState_read) ENABLED START #
        """Return the cspCbfState attribute."""
        return tango.DevState.UNKNOWN
        # PROTECTED REGION END #    //  CspMaster.cspCbfState_read

    def read_cspPssState(self):
        # PROTECTED REGION ID(CspMaster.cspPssState_read) ENABLED START #
        """Return the cspPssState attribute."""
        return tango.DevState.UNKNOWN
        # PROTECTED REGION END #    //  CspMaster.cspPssState_read

    def read_cspPstState(self):
        # PROTECTED REGION ID(CspMaster.cspPstState_read) ENABLED START #
        """Return the cspPstState attribute."""
        return tango.DevState.UNKNOWN
        # PROTECTED REGION END #    //  CspMaster.cspPstState_read

    def read_cspCbfHealthState(self):
        # PROTECTED REGION ID(CspMaster.cspCbfHealthState_read) ENABLED START #
        """Return the cspCbfHealthState attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.cspCbfHealthState_read

    def read_cspPssHealthState(self):
        # PROTECTED REGION ID(CspMaster.cspPssHealthState_read) ENABLED START #
        """Return the cspPssHealthState attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.cspPssHealthState_read

    def read_cspPstHealthState(self):
        # PROTECTED REGION ID(CspMaster.cspPstHealthState_read) ENABLED START #
        """Return the cspPstHealthState attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.cspPstHealthState_read

    def read_cbfMasterAddress(self):
        # PROTECTED REGION ID(CspMaster.cbfMasterAddress_read) ENABLED START #
        """Return the cbfMasterAddress attribute."""
        return ''
        # PROTECTED REGION END #    //  CspMaster.cbfMasterAddress_read

    def read_pssMasterAddress(self):
        # PROTECTED REGION ID(CspMaster.pssMasterAddress_read) ENABLED START #
        """Return the pssMasterAddress attribute."""
        return ''
        # PROTECTED REGION END #    //  CspMaster.pssMasterAddress_read

    def read_pstMasterAddress(self):
        # PROTECTED REGION ID(CspMaster.pstMasterAddress_read) ENABLED START #
        """Return the pstMasterAddress attribute."""
        return ''
        # PROTECTED REGION END #    //  CspMaster.pstMasterAddress_read

    def read_cspCbfAdminMode(self):
        # PROTECTED REGION ID(CspMaster.cspCbfAdminMode_read) ENABLED START #
        """Return the cspCbfAdminMode attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.cspCbfAdminMode_read

    def write_cspCbfAdminMode(self, value):
        # PROTECTED REGION ID(CspMaster.cspCbfAdminMode_write) ENABLED START #
        """Set the cspCbfAdminMode attribute."""
        pass
        # PROTECTED REGION END #    //  CspMaster.cspCbfAdminMode_write

    def read_cspPssAdminMode(self):
        # PROTECTED REGION ID(CspMaster.cspPssAdminMode_read) ENABLED START #
        """Return the cspPssAdminMode attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.cspPssAdminMode_read

    def write_cspPssAdminMode(self, value):
        # PROTECTED REGION ID(CspMaster.cspPssAdminMode_write) ENABLED START #
        """Set the cspPssAdminMode attribute."""
        pass
        # PROTECTED REGION END #    //  CspMaster.cspPssAdminMode_write

    def read_cspPstAdminMode(self):
        # PROTECTED REGION ID(CspMaster.cspPstAdminMode_read) ENABLED START #
        """Return the cspPstAdminMode attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.cspPstAdminMode_read

    def write_cspPstAdminMode(self, value):
        # PROTECTED REGION ID(CspMaster.cspPstAdminMode_write) ENABLED START #
        """Set the cspPstAdminMode attribute."""
        pass
        # PROTECTED REGION END #    //  CspMaster.cspPstAdminMode_write

    def read_unassignedSearchBeamsNum(self):
        # PROTECTED REGION ID(CspMaster.unassignedSearchBeamsNum_read) ENABLED START #
        """Return the unassignedSearchBeamsNum attribute."""
        return 0
        # PROTECTED REGION END #    //  CspMaster.unassignedSearchBeamsNum_read

    def read_availableCapabilities(self):
        # PROTECTED REGION ID(CspMaster.availableCapabilities_read) ENABLED START #
        """Return the availableCapabilities attribute."""
        return ('',)
        # PROTECTED REGION END #    //  CspMaster.availableCapabilities_read

    def read_reportSearchBeamState(self):
        # PROTECTED REGION ID(CspMaster.reportSearchBeamState_read) ENABLED START #
        """Return the reportSearchBeamState attribute."""
        return (tango.DevState.UNKNOWN,)
        # PROTECTED REGION END #    //  CspMaster.reportSearchBeamState_read

    def read_reportSearchBeamHealthState(self):
        # PROTECTED REGION ID(CspMaster.reportSearchBeamHealthState_read) ENABLED START #
        """Return the reportSearchBeamHealthState attribute."""
        return (0,)
        # PROTECTED REGION END #    //  CspMaster.reportSearchBeamHealthState_read

    def read_reportSearchBeamAdminMode(self):
        # PROTECTED REGION ID(CspMaster.reportSearchBeamAdminMode_read) ENABLED START #
        """Return the reportSearchBeamAdminMode attribute."""
        return (0,)
        # PROTECTED REGION END #    //  CspMaster.reportSearchBeamAdminMode_read

    def write_reportSearchBeamAdminMode(self, value):
        # PROTECTED REGION ID(CspMaster.reportSearchBeamAdminMode_write) ENABLED START #
        """Set the reportSearchBeamAdminMode attribute."""
        pass
        # PROTECTED REGION END #    //  CspMaster.reportSearchBeamAdminMode_write

    def read_reportTimingBeamState(self):
        # PROTECTED REGION ID(CspMaster.reportTimingBeamState_read) ENABLED START #
        """Return the reportTimingBeamState attribute."""
        return (tango.DevState.UNKNOWN,)
        # PROTECTED REGION END #    //  CspMaster.reportTimingBeamState_read

    def read_reportTimingBeamHealthState(self):
        # PROTECTED REGION ID(CspMaster.reportTimingBeamHealthState_read) ENABLED START #
        """Return the reportTimingBeamHealthState attribute."""
        return (0,)
        # PROTECTED REGION END #    //  CspMaster.reportTimingBeamHealthState_read

    def read_reportTimingBeamAdminMode(self):
        # PROTECTED REGION ID(CspMaster.reportTimingBeamAdminMode_read) ENABLED START #
        """Return the reportTimingBeamAdminMode attribute."""
        return (0,)
        # PROTECTED REGION END #    //  CspMaster.reportTimingBeamAdminMode_read

    def write_reportTimingBeamAdminMode(self, value):
        # PROTECTED REGION ID(CspMaster.reportTimingBeamAdminMode_write) ENABLED START #
        """Set the reportTimingBeamAdminMode attribute."""
        pass
        # PROTECTED REGION END #    //  CspMaster.reportTimingBeamAdminMode_write

    def read_reportVlbiBeamState(self):
        # PROTECTED REGION ID(CspMaster.reportVlbiBeamState_read) ENABLED START #
        """Return the reportVlbiBeamState attribute."""
        return (tango.DevState.UNKNOWN,)
        # PROTECTED REGION END #    //  CspMaster.reportVlbiBeamState_read

    def read_reportVlbiBeamHealthState(self):
        # PROTECTED REGION ID(CspMaster.reportVlbiBeamHealthState_read) ENABLED START #
        """Return the reportVlbiBeamHealthState attribute."""
        return (0,)
        # PROTECTED REGION END #    //  CspMaster.reportVlbiBeamHealthState_read

    def read_reportVlbiBeamAdminMode(self):
        # PROTECTED REGION ID(CspMaster.reportVlbiBeamAdminMode_read) ENABLED START #
        """Return the reportVlbiBeamAdminMode attribute."""
        return (0,)
        # PROTECTED REGION END #    //  CspMaster.reportVlbiBeamAdminMode_read

    def write_reportVlbiBeamAdminMode(self, value):
        # PROTECTED REGION ID(CspMaster.reportVlbiBeamAdminMode_write) ENABLED START #
        """Set the reportVlbiBeamAdminMode attribute."""
        pass
        # PROTECTED REGION END #    //  CspMaster.reportVlbiBeamAdminMode_write

    def read_cspSubarrayAddresses(self):
        # PROTECTED REGION ID(CspMaster.cspSubarrayAddresses_read) ENABLED START #
        """Return the cspSubarrayAddresses attribute."""
        return ('',)
        # PROTECTED REGION END #    //  CspMaster.cspSubarrayAddresses_read

    def read_searchBeamAddresses(self):
        # PROTECTED REGION ID(CspMaster.searchBeamAddresses_read) ENABLED START #
        """Return the searchBeamAddresses attribute."""
        return ('',)
        # PROTECTED REGION END #    //  CspMaster.searchBeamAddresses_read

    def read_timingBeamAddresses(self):
        # PROTECTED REGION ID(CspMaster.timingBeamAddresses_read) ENABLED START #
        """Return the timingBeamAddresses attribute."""
        return ('',)
        # PROTECTED REGION END #    //  CspMaster.timingBeamAddresses_read

    def read_vlbiBeamAddresses(self):
        # PROTECTED REGION ID(CspMaster.vlbiBeamAddresses_read) ENABLED START #
        """Return the vlbiBeamAddresses attribute."""
        return ('',)
        # PROTECTED REGION END #    //  CspMaster.vlbiBeamAddresses_read

    def read_searchBeamMembership(self):
        # PROTECTED REGION ID(CspMaster.searchBeamMembership_read) ENABLED START #
        """Return the searchBeamMembership attribute."""
        return (0,)
        # PROTECTED REGION END #    //  CspMaster.searchBeamMembership_read

    def read_timingBeamMembership(self):
        # PROTECTED REGION ID(CspMaster.timingBeamMembership_read) ENABLED START #
        """Return the timingBeamMembership attribute."""
        return (0,)
        # PROTECTED REGION END #    //  CspMaster.timingBeamMembership_read

    def read_vlbiBeamMembership(self):
        # PROTECTED REGION ID(CspMaster.vlbiBeamMembership_read) ENABLED START #
        """Return the vlbiBeamMembership attribute."""
        return (0,)
        # PROTECTED REGION END #    //  CspMaster.vlbiBeamMembership_read

    def read_unassignedSearchBeamIDs(self):
        # PROTECTED REGION ID(CspMaster.unassignedSearchBeamIDs_read) ENABLED START #
        """Return the unassignedSearchBeamIDs attribute."""
        return (0,)
        # PROTECTED REGION END #    //  CspMaster.unassignedSearchBeamIDs_read

    def read_reservedSearchBeamIDs(self):
        # PROTECTED REGION ID(CspMaster.reservedSearchBeamIDs_read) ENABLED START #
        """Return the reservedSearchBeamIDs attribute."""
        return (0,)
        # PROTECTED REGION END #    //  CspMaster.reservedSearchBeamIDs_read

    def read_reservedSearchBeamNum(self):
        # PROTECTED REGION ID(CspMaster.reservedSearchBeamNum_read) ENABLED START #
        """Return the reservedSearchBeamNum attribute."""
        return (0,)
        # PROTECTED REGION END #    //  CspMaster.reservedSearchBeamNum_read

    def read_unassignedTimingBeamIDs(self):
        # PROTECTED REGION ID(CspMaster.unassignedTimingBeamIDs_read) ENABLED START #
        """Return the unassignedTimingBeamIDs attribute."""
        return (0,)
        # PROTECTED REGION END #    //  CspMaster.unassignedTimingBeamIDs_read

    def read_unassignedVlbiBeamIDs(self):
        # PROTECTED REGION ID(CspMaster.unassignedVlbiBeamIDs_read) ENABLED START #
        """Return the unassignedVlbiBeamIDs attribute."""
        return (0,)
        # PROTECTED REGION END #    //  CspMaster.unassignedVlbiBeamIDs_read


    # --------
    # Commands
    # --------

    @command(
        dtype_in='DevVarStringArray',
        doc_in=("If the array length is 0, the command applies to the whole CSP Element."
                " If the array length is > 1, each array element specifies the FQDN of "
                "the CSP SubElement to switch ON."),
    )
    @DebugIt()
    def On(self, argin):
        # PROTECTED REGION ID(CspMaster.On) ENABLED START #
        """
        *Class TANGO method*
        
        Switch-on the CSP sub-elements specified by the input argument. If no argument is
        specified, the command is issued on all the CSP sub-elements.
        The command is executed if the *AdminMode* is ONLINE or *MAINTENANCE*.
        If the AdminMode is *OFFLINE*, *NOT-FITTED* or *RESERVED*, the method throws an 
        exception.

        :param argin: 
                    The list of sub-element FQDNs to switch-on or an empty list to switch-on the whole 
                    CSP Element.
                    
                    If the array length is 0, the command applies to the whole CSP Element. If the 
                    array length is > 1, each array element specifies the FQDN of the
                    CSP SubElement to switch ON.
        :type: 'DevVarStringArray'                    
        :return: None
        :raises: *tango.DevFailed* exception.\n     
                The exception is thrown when the *On* command fails
                for CBF sub-element or there are no DeviceProxy providing interface
                to the CSP sub-elements or the AdminMode is not correct.
        """
        pass
        # PROTECTED REGION END #    //  CspMaster.On

    @command(
        dtype_in='DevVarStringArray',
        doc_in=("If the array length is 0, the command applies to the whole CSP Element."
                "If the array length is > 1, each array element specifies the FQDN of the"
                " CSP SubElement to switch OFF."),
    )
    @DebugIt()
    def Off(self, argin):
        # PROTECTED REGION ID(CspMaster.Off) ENABLED START #
        """
        Switch-off the CSP sub-elements specified by the input argument. 
        If no argument is specified, the command is issued to all the CSP 
        sub-elements.

        :param argin:         
        
            If the array length is 0, the command applies to the whole CSP Element.
            If the array length is > 1, each array element specifies the FQDN of the
            CSP SubElement to switch OFF.
        :type: 'DevVarStringArray'

        :return: None
        """
        pass
        # PROTECTED REGION END #    //  CspMaster.Off

    @command(
        dtype_in='DevVarStringArray',
        doc_in=("If the array length is 0, the command applies to the whole CSP Element. "
                "If the array length is > 1, each array element specifies the FQDN of the"
                "CSP SubElement to put in STANDBY mode."),
    )
    @DebugIt()
    def Standby(self, argin):
        # PROTECTED REGION ID(CspMaster.Standby) ENABLED START #
        """
        
            Transit CSP or one or more CSP SubElements from ON/OFF to 
            STANDBY.

        :param argin: 'DevVarStringArray'
        
            If the array length is 0, the command applies to the whole
            CSP Element.
            If the array length is > 1, each array element specifies the FQDN of the
            CSP SubElement to put in STANDBY mode.

        :return: None
        """
        pass
        # PROTECTED REGION END #    //  CspMaster.Standby

    @command(
    )
    @DebugIt()
    def Upgrade(self):
        # PROTECTED REGION ID(CspMaster.Upgrade) ENABLED START #
        """

        :return: None
        """
        pass
        # PROTECTED REGION END #    //  CspMaster.Upgrade

# ----------
# Run server
# ----------


def main(args=None, **kwargs):
    # PROTECTED REGION ID(CspMaster.main) ENABLED START #
    return run((CspMaster,), args=args, **kwargs)
    # PROTECTED REGION END #    //  CspMaster.main

if __name__ == '__main__':
    main()
